from dexy.version import DEXY_VERSION
import datetime
import dexy.exceptions

# TODO list available citation types

def cite_command(
        fmt='bibtex' # desired format of citation
        ):
    """
    How to cite dexy in papers.
    """
    if fmt == 'bibtex':
        cite_bibtex()
    else:
        msg = "Don't know how to provide citation in '%s' format"
        raise dexy.exceptions.UserFeedback(msg % fmt)

def bibtex_text():
    args = {
            'version' : DEXY_VERSION,
            'year' : datetime.date.today().year
            }

    return """@misc{Dexy,
    title = {Dexy: Reproducible Data Analysis and Document Automation Software, Version~%(version)s},
    author = {{Nelson, Ana}},
    year = {%(year)s},
    url = {http://www.dexy.it/},
    note = {http://orcid.org/0000-0003-2561-1564}
}""" % args

def cite_bibtex():
    print bibtex_text()



from bs4 import BeautifulSoup
from dexy.filter import DexyFilter
import inflection
import re

class Customize(DexyFilter):
    """
    Add <script> tags or <link> tags to an HTML file's header.
    
    Uses BeautifulSoup.
    """
    aliases = ['customize']

    _settings = {
            'scripts' : ("Javascript files to add.", []),
            'stylesheets' : ("CSS files to add.", [])
            }

    def process_text(self, input_text):
        soup = BeautifulSoup(input_text)

        for js in self.setting('scripts'):
            js_tag = soup.new_tag("script", type="text/javascript", src=js)
            soup.head.append(js_tag)

        for css in self.setting('stylesheets'):
            css_tag = soup.new_tag("link", rel="stylesheet", type="text/css", href=css)
            soup.head.append(css_tag)

        return unicode(soup)

class SoupSections(DexyFilter):
    """
    Split a HTML file into nested sections based on header tags.
    """
    aliases = ['soups']

    _settings = {
            'data-type' : 'sectioned',
            'html-parser' : ("Name of html parser BeautifulSoup should use.", 'html.parser'),
            'initial-section-name' : ("Name to use for the initial section which currently holds all the contents.", u"Actual Document Contents"),
            }

    def append_current_section(self):
        section_dict = {
                "name" : self.current_section_name,
                "contents" : self.current_section_text,
                "level" : self.current_section_level,
                "id" : self.current_section_anchor
                }
        self.output_data._data.append(section_dict)

    def process(self):
        soup = BeautifulSoup(unicode(self.input_data), self.setting('html-parser'))

        for tag in soup.find_all(re.compile("^h[0-6]")):
            name = tag.text
            m = re.match("^h([0-6])$", tag.name)

            if not tag.attrs.has_key('id'):
                tag.attrs['id'] = inflection.parameterize(name)

            self.current_section_anchor = tag.attrs['id']
            self.current_section_text = None
            self.current_section_name = name
            self.current_section_level = int(m.groups()[0])

            self.append_current_section()

        self.current_section_text = unicode(soup)
        self.current_section_name = self.setting('initial-section-name')
        self.current_section_level = 1
        self.current_section_anchor = None

        self.append_current_section()

        self.output_data.save()



from dexy.utils import defaults
from dexy.commands.utils import dummy_wrapper
from dexy.parser import AbstractSyntaxTree
from dexy.parser import Parser

def parsers_command():
    wrapper = dummy_wrapper()
    ast = AbstractSyntaxTree(wrapper)

    processed_aliases = set()

    for alias in sorted(Parser.plugins):
        if alias in processed_aliases:
            continue

        parser = Parser.create_instance(alias, ast, wrapper)

        for alias in parser.aliases:
            processed_aliases.add(alias)

        print "%s Parser" % parser.__class__.__name__
        print ''
        print parser.setting('help')
        print ''
        print "aliases:"
        for alias in parser.aliases:
            print "  %s" % alias
        print ''

    print "Default parsers are: " + defaults['parsers']
    print ''
    print "Dexy will only look for config files to parse in the root directory"
    print "of your project unless --recurse is specified."
    print ''



from tests.utils import assert_output_matches
from mock import patch, call
from StringIO import StringIO


# See http://www.voidspace.org.uk/python/mock/patch.html#where-to-patch for
# why this mock applies to dexy.filters.websequence not urllib2
@patch("dexy.filters.websequence.urlopen",
       side_effect=[StringIO("?png=whereToGo"), StringIO("mock_result")])
def test_websequence_svg(mocked_urlopen):
    assert_output_matches("wsd", "Sender->Receiver", "mock_result", ext=".wsd")
    print mocked_urlopen.call_args_list
    assert mocked_urlopen.call_args_list[0] == call(
        "http://www.websequencediagrams.com/index.php",
        'message=Sender-%3EReceiver&style=default&apiVersion=1')
    assert mocked_urlopen.call_args_list[1] == call(
        "http://www.websequencediagrams.com/?png=whereToGo")
